/**
   In our textbook, the distinction between the Node class
   and the LinkedList class is very blurred. In the textbook,
   the Node class takes on much of the reponsibility of
   the LinkedList class.

   This file shows how the Node class should be used to
   implement the LinkedList class. The Node class itself
   is really a "private" class that should only be known
   to the LinkedList class.
*/


/**
   This class is used to implement the LinkedList class.
   It is not really to be used by itself.

   This should be a private, nested class inside of LinkedList.
*/
class Node
{
   public int  data;
   public Node next;

   public Node(int data, Node next)
   {
      this.data = data;
      this.next = next;
   }
}//Node


/**
   A simple implementation of the "List" abstract data type.
*/
class LinkedList
{
   private Node first;

   public LinkedList()
   {
      first = null;
   }

   public int length()
   {
      Node cursor = first;
      int length = 0;

      while (cursor != null)
      {
         length++;
         cursor = cursor.next;
      }
      return length;
   }

   public boolean empty()
   {
      return (first == null);
   }

   public int getFirst()
   {
      return first.data;
   }

   public void insert(int data)
   {
      Node node = new Node(data, first);
      first = node;
   }

   public int remove()
   {
      int data = first.data;
      first = first.next;
      return data;
   }

   public String toString()
   {
      String result = "[ ";
      Node cursor = first;
      while ( cursor != null )
      {
         result += cursor.data + " ";
         cursor = cursor.next;
      }
      result += "]";
      return result;
   }

}//LinkedList



/**
   A simple test of the LinkedList data type.
*/
public class LinkedList_vs_Node
{
   public static void main(String[] args)
   {
      LinkedList myList = new LinkedList();

      myList.insert(1);
      myList.insert(2);
      myList.insert(3);
      myList.insert(4);
      myList.insert(5);
      myList.insert(6);

      System.out.println( "length = " + myList.length() );

      System.out.println( myList );

      while ( ! myList.empty() )
      {
         System.out.println( myList.getFirst() );
         myList.remove();
      }
   }//main()
}//LinkedList_vs_Node
